import Flutter
import UIKit
import TalsecRuntime

public class SwiftFreeraspPlugin: NSObject, FlutterPlugin, FlutterStreamHandler {
  
    //Channels to pass data
    static var configChannel: FlutterMethodChannel?
    static var eventChannel: FlutterEventChannel?
    static var eventList: Array<String> = []
    static var event: FlutterEventSink?
    static var messenger: FlutterBinaryMessenger?
    
    public static func register(with registrar: FlutterPluginRegistrar) {
    messenger = registrar.messenger()
    
    //Channels init
    configChannel = FlutterMethodChannel(name: "plugins.aheaditec.com/config", binaryMessenger: messenger!)
    eventChannel = FlutterEventChannel(name: "plugins.aheaditec.com/events", binaryMessenger: messenger!)
        
    let instance = SwiftFreeraspPlugin()
    registrar.addMethodCallDelegate(instance, channel: configChannel!)
  }

  public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
    switch call.method {
    case "getConfig":
        let args = call.arguments as! Dictionary<String, String>
        let abi = args["appBundleId"]! as String
        let ati = args["appTeamId"]! as String
        let wm = args["watcherMail"]! as String
        let config = TalsecConfig(appBundleId: abi, appTeamId: ati, watcherMailAddress: wm)
        SwiftFreeraspPlugin.eventChannel?.setStreamHandler(self)
        Talsec.start(config: config)
    default:
        print("Unimplemeted state")
    }
  }
    
    public func onListen(withArguments arguments: Any?, eventSink events: @escaping FlutterEventSink) -> FlutterError? {
        SwiftFreeraspPlugin.event = events
        while !SwiftFreeraspPlugin.eventList.isEmpty {
            events(SwiftFreeraspPlugin.eventList.popLast())
        }
        return nil
    }

    public func onCancel(withArguments arguments: Any?) -> FlutterError? {
        SwiftFreeraspPlugin.event = nil
        return nil
    }
}

extension SecurityThreatCenter: SecurityThreatHandler {

    func invokeCallback(callback: String){
        if SwiftFreeraspPlugin.event == nil {
            SwiftFreeraspPlugin.eventList.append(callback)
            
        } else {
            SwiftFreeraspPlugin.event!(callback)
        }
    }

    public func threatDetected(_ securityThreat: TalsecRuntime.SecurityThreat) {
        switch securityThreat {
        case SecurityThreat.signature:
            invokeCallback(callback: "onSignatureDetected")
        case SecurityThreat.jailbreak:
            invokeCallback(callback: "onJailbreakDetected")
        case SecurityThreat.debugger:
            invokeCallback(callback: "onDebuggerDetected")
        case SecurityThreat.runtimeManipulation:
            invokeCallback(callback: "onRuntimeManipulationDetected")
        case SecurityThreat.passcode:
            invokeCallback(callback: "onPasscodeDetected")
        case SecurityThreat.passcodeChange:
            invokeCallback(callback: "onPasscodeChangeDetected")
        case SecurityThreat.simulator:
            invokeCallback(callback: "onSimulatorDetected")
        case SecurityThreat.missingSecureEnclave:
            invokeCallback(callback: "onMissingSecureEnclaveDetected")
        @unknown default:
            print("Undefined threat callback")
        }
    }
}

